@@ -36,7 +36,7 @@ from yd_commands.variables import (
     add_substitutions,
     load_toml_file_with_variable_substitutions,
     process_variable_substitutions,
-    process_variable_substitutions_in_dict_insitu,
+    process_variable_substitutions_insitu,
 )
 
 # CLI > YD_CONF > 'config.toml'
@@ -172,7 +172,7 @@ def load_config_work_requirement() -> Optional[ConfigWorkRequirement]:
     # Process any new substitutions after the common config
     # has been processed
     for _ in range(TOML_VAR_NESTED_DEPTH):
-        process_variable_substitutions_in_dict_insitu(wr_section)
+        process_variable_substitutions_insitu(wr_section)
 
     try:
         # Allow WORKER_TAG if WORKER_TAGS is empty
@@ -270,6 +270,7 @@ def load_config_work_requirement() -> Optional[ConfigWorkRequirement]:
             task_data_file=wr_section.get(TASK_DATA_FILE, None),
             task_group_name=wr_section.get(TASK_GROUP_NAME, None),
             task_name=wr_section.get(TASK_NAME, None),
+            task_timeout=wr_section.get(TASK_TIMEOUT, None),
             task_type=task_type,
             tasks_per_worker=wr_section.get(TASKS_PER_WORKER, None),
             upload_files=wr_section.get(UPLOAD_FILES, []),
@@ -301,8 +302,8 @@ def load_config_worker_pool() -> Optional[ConfigWorkerPool]:
     # Process any new substitutions after the common config
     # has been processed
     for _ in range(TOML_VAR_NESTED_DEPTH):
-        process_variable_substitutions_in_dict_insitu(wp_section)
-        process_variable_substitutions_in_dict_insitu(cr_section)
+        process_variable_substitutions_insitu(wp_section)
+        process_variable_substitutions_insitu(cr_section)
 
     duplicate_keys = set(wp_section.keys()).intersection(set(cr_section.keys()))
     if len(duplicate_keys) != 0:
