@@ -87,7 +87,7 @@ from yd_commands.variables import (
     load_json_file_with_variable_substitutions,
     load_jsonnet_file_with_variable_substitutions,
     load_toml_file_with_variable_substitutions,
-    process_variable_substitutions_in_dict_insitu,
+    process_variable_substitutions_insitu,
 )
 from yd_commands.wrapper import ARGS_PARSER, CLIENT, CONFIG_COMMON, main_wrapper
 
@@ -236,7 +236,7 @@ def submit_work_requirement(
     # Re-process substitutions in the CONFIG_WR object
     CONFIG_WR = update_config_work_requirement_object(CONFIG_WR)
     # Re-process substitutions in the wr_data dictionary
-    process_variable_substitutions_in_dict_insitu(wr_data)
+    process_variable_substitutions_insitu(wr_data)
 
     # Handle any files that need to be uploaded
     global UPLOADED_FILES
@@ -364,7 +364,7 @@ def create_task_group(
         L_TASK_GROUP_NUMBER, formatted_number_str(tg_number, num_task_groups)
     )
     add_substitution_overwrite(L_TASK_GROUP_COUNT, str(num_task_groups))
-    process_variable_substitutions_in_dict_insitu(task_group_data)
+    process_variable_substitutions_insitu(task_group_data)
     # Create a copy of global CONFIG_WR and apply lazy substitutions
     config_wr = update_config_work_requirement_object(deepcopy(CONFIG_WR))
 
@@ -407,6 +407,14 @@ def create_task_group(
         if providers_data is None
         else [CloudProvider(provider) for provider in providers_data]
     )
+    task_timeout_minutes: Optional[float] = check_float_or_int(
+        task_group_data.get(TASK_TIMEOUT, config_wr.task_timeout)
+    )
+    task_timeout: Optional[timedelta] = (
+        None
+        if task_timeout_minutes is None
+        else timedelta(minutes=task_timeout_minutes)
+    )
 
     run_specification = RunSpecification(
         taskTypes=task_types,
@@ -442,6 +450,7 @@ def create_task_group(
         regions=check_list(
             task_group_data.get(REGIONS, wr_data.get(REGIONS, config_wr.regions))
         ),
+        taskTimeout=task_timeout,
     )
     ctttl_data = check_float_or_int(
         task_group_data.get(
@@ -577,7 +586,7 @@ def add_tasks_to_task_group(
             add_substitution_overwrite(
                 L_TASK_NUMBER, formatted_number_str(task_number, num_tasks)
             )
-            process_variable_substitutions_in_dict_insitu(task)
+            process_variable_substitutions_insitu(task)
             config_wr = update_config_work_requirement_object(deepcopy(CONFIG_WR))
 
             executable = check_str(
@@ -598,6 +607,15 @@ def add_tasks_to_task_group(
                 task.get(ENV, task_group_data.get(ENV, wr_data.get(ENV, config_wr.env)))
             )
 
+            # Task timeout is automatically inherited from the Task Group level
+            # unless overridden by the Task
+            task_timeout_minutes = check_float_or_int(task.get(TASK_LEVEL_TIMEOUT, None))
+            task_timeout = (
+                None
+                if task_timeout_minutes is None
+                else timedelta(minutes=task_timeout_minutes)
+            )
+
             # Set up lists of files to input, verify
             input_files_list = check_list(
                 task.get(
@@ -800,6 +818,7 @@ def add_tasks_to_task_group(
                     env=env,
                     inputs=inputs,
                     outputs=outputs,
+                    task_timeout=task_timeout,
                     flatten_upload_paths=flatten_upload_paths,
                 )
             )
@@ -863,9 +882,24 @@ def get_task_data_property(
                 f"Task '{task_name}': Properties '{TASK_DATA}' and "
                 f"'{TASK_DATA_FILE}' are both set"
             )
+
         if task_data_property:
             return task_data_property
+
         if task_data_file_property:
+            # Switch to the directory containing the config file; will use the current
+            # directory if the config file is absent
+            source_directory = (
+                CONFIG_FILE_DIR
+                if ARGS_PARSER.content_path is None or ARGS_PARSER.content_path == ""
+                else ARGS_PARSER.content_path
+            )
+            try:
+                chdir(source_directory)
+            except Exception as e:
+                raise Exception(
+                    f"Unable to switch to content directory '{source_directory}': {e}"
+                )
             with open(task_data_file_property, "r") as f:
                 return f.read()
 
@@ -1043,9 +1077,6 @@ def get_task_group_name(
     return name
 
 
-PRINT_NO_EXECUTABLE_MESSAGE = True
-
-
 def create_task(
     config_wr: ConfigWorkRequirement,
     wr_data: Dict,
@@ -1062,6 +1093,7 @@ def create_task(
     env: Dict[str, str],
     inputs: Optional[List[TaskInput]],
     outputs: Optional[List[TaskOutput]],
+    task_timeout: Optional[timedelta],
     flatten_upload_paths: bool = False,
 ) -> Task:
     """
@@ -1087,6 +1119,7 @@ def create_task(
             outputs=outputs,
             flattenInputPaths=flatten_input_paths,
             taskData=task_data_property,
+            timeout=task_timeout,
         )
 
     # Flatten paths for downloaded files?
@@ -1115,12 +1148,10 @@ def create_task(
     # and added to the list of required files.
     if task_type in ["bash", "powershell", "python", "cmd", "bat"]:
         if executable is None:
-            global PRINT_NO_EXECUTABLE_MESSAGE
-            if PRINT_NO_EXECUTABLE_MESSAGE:
+            if task_number == 1:  # Only report this the first time
                 print_log(
                     f"Note: no 'executable' specified for '{task_type}' Task Type"
                 )
-                PRINT_NO_EXECUTABLE_MESSAGE = False
             return _make_task(flatten_input_paths)
 
         UPLOADED_FILES.add_input_file(
@@ -1239,7 +1270,7 @@ def submit_json_raw(wr_file: str):
     wr_data["name"] = format_yd_name(wr_data["name"])
     wr_name = wr_data["name"]
     add_substitutions(subs={L_WR_NAME: wr_name})
-    process_variable_substitutions_in_dict_insitu(wr_data)
+    process_variable_substitutions_insitu(wr_data)
 
     if ARGS_PARSER.dry_run:
         # This will show the results of any variable substitutions
