@@ -5,19 +5,17 @@ Utility functions for use with the submit command.
 import re
 from dataclasses import dataclass
 from glob import glob
+from time import sleep
 from typing import List, Optional
 
 from yellowdog_client import PlatformClient
 from yellowdog_client.model import ObjectPath, TaskInput, TaskInputVerification
 
-from yd_commands.config_types import (
-    NAMESPACE_SEPARATOR,
-    ConfigCommon,
-    ConfigWorkRequirement,
-)
+from yd_commands.config_types import ConfigCommon, ConfigWorkRequirement
 from yd_commands.printing import print_error, print_log
+from yd_commands.settings import NAMESPACE_PREFIX_SEPARATOR
 from yd_commands.upload_utils import unique_upload_pathname, upload_file_core
-from yd_commands.variables import substitute_variable_str
+from yd_commands.variables import process_variable_substitutions_insitu
 from yd_commands.wrapper import ARGS_PARSER
 
 
@@ -62,7 +60,7 @@ def get_namespace_and_filepath(
     Find the namespace and path, using the namespace separator.
     """
     try:
-        file_parts = file.split(NAMESPACE_SEPARATOR)
+        file_parts = file.split(NAMESPACE_PREFIX_SEPARATOR)
 
         if len(file_parts) == 1:
             # Use the Work Requirement ID, if supplied, as the directory
@@ -157,7 +155,7 @@ class UploadedFiles:
             else:
                 print_log(
                     f"Dry-run: Would upload '{upload_file}' to"
-                    f" '{namespace}::{uploaded_file_path}'"
+                    f" '{namespace}{NAMESPACE_PREFIX_SEPARATOR}{uploaded_file_path}'"
                 )
 
             self._uploaded_files.append(
@@ -187,7 +185,9 @@ class UploadedFiles:
                 flatten_upload_paths=flatten_upload_paths,
             )
             # Force 'uploaded_file_name' to be at the root of the namespace
-            self.add_upload_file(filename, f"{NAMESPACE_SEPARATOR}{upload_file_name}")
+            self.add_upload_file(
+                filename, f"{NAMESPACE_PREFIX_SEPARATOR}{upload_file_name}"
+            )
 
         return expanded_files
 
@@ -220,19 +220,72 @@ def format_yd_name(yd_name: str) -> str:
     Format a string to be consistent with YellowDog naming requirements.
     """
     # Make obvious substitutions
-    yd_name = yd_name.replace("/", "-").replace(" ", "_").lower()
-    # Enforce acceptable regex and name length
-    yd_name = re.sub("[^a-z0-9_-]", "", yd_name)
-    if not yd_name[0].isalpha():
-        yd_name = f"yd_{yd_name}"
-    return yd_name[:60]
+    new_yd_name = yd_name.replace("/", "-").replace(" ", "_").lower()
+    # Enforce acceptable regex, starting character and name length
+    new_yd_name = re.sub("[^a-z0-9_-]", "", new_yd_name)
+    if not new_yd_name[0].isalpha():
+        new_yd_name = f"yd_{new_yd_name}"
+    new_yd_name = new_yd_name[:60]
+    if new_yd_name != yd_name:
+        print_log(
+            f"Changing name '{yd_name}' to '{new_yd_name}' to comply with YellowDog"
+            " naming requirements"
+        )
+    return new_yd_name
 
 
-def update_config_work_requirement(config_wr: ConfigWorkRequirement):
+def update_config_work_requirement_object(
+    config_wr: ConfigWorkRequirement,
+) -> ConfigWorkRequirement:
     """
-    Update a ConfigWorkRequirement Object with the current dictionary of
+    Update a ConfigWorkRequirement Object using the current
     variable substitutions. Returns the updated object.
     """
-    config_wr_str_processed = substitute_variable_str(str(config_wr))
-    # Note: 'literal_eval' doesn't work here
-    return eval(config_wr_str_processed)
+    config_wr_dict = config_wr.__dict__
+    process_variable_substitutions_insitu(config_wr_dict)
+    return ConfigWorkRequirement(**config_wr_dict)
+
+
+def pause_between_batches(task_batch_size: int, batch_number: int, num_tasks: int):
+    """
+    Process a pause between Task batches.
+    """
+    if ARGS_PARSER.pause_between_batches is None:
+        return
+
+    first_batch: bool = batch_number == 0
+    task_num_start = (task_batch_size * batch_number) + 1
+    task_num_end = min(task_batch_size * (batch_number + 1), num_tasks)
+    task_range_str = (
+        f"Tasks {task_num_start}-{task_num_end}"
+        if task_num_start != task_num_end
+        else f"Task {task_num_start}"
+    )
+
+    if ARGS_PARSER.pause_between_batches <= 0:  # Manual delay
+        print_log(
+            (
+                f"Submitting batch number {batch_number + 1} ({task_range_str})"
+                if first_batch
+                else (
+                    "Pausing before submitting batch number"
+                    f" {batch_number + 1} ({task_range_str}). Press enter to continue:"
+                )
+            ),
+            override_quiet=True,
+        )
+        if not first_batch:
+            input()
+
+    elif ARGS_PARSER.pause_between_batches > 0:  # Automatic delay
+        print_log(
+            f"Submitting batch number {batch_number + 1} ({task_range_str})"
+            if first_batch
+            else (
+                f"Pausing for {ARGS_PARSER.pause_between_batches} seconds before"
+                f" submitting batch number {batch_number + 1}"
+                f" ({task_range_str})"
+            )
+        )
+        if not first_batch:
+            sleep(ARGS_PARSER.pause_between_batches)
